import bpy

class BakeProperties(bpy.types.PropertyGroup):
    use_transparency: bpy.props.BoolProperty(name="Alpha", description="Bake with transparency", default=False)

class BakeTextureOperator(bpy.types.Operator):
    bl_idname = "object.bake_texture"
    bl_label = "Bake ID to Texture"
    bl_description = "Bake the ID colors into a Texture"

    def execute(self, context):
        def set_temp_material(obj):
            temp_materials = []
            for slot in obj.material_slots:
                orig_mat = slot.material
                temp_mat = bpy.data.materials.new(name=orig_mat.name + "_temp")
                temp_mat.use_nodes = True
                nodes = temp_mat.node_tree.nodes
                principled_bsdf = nodes.get('Principled BSDF')
                principled_bsdf.inputs['Base Color'].default_value = orig_mat.diffuse_color
                slot.material = temp_mat
                temp_materials.append((orig_mat, temp_mat))
            return temp_materials

        def restore_materials(obj, materials):
            for slot, (orig_mat, temp_mat) in zip(obj.material_slots, materials):
                slot.material = orig_mat
                bpy.data.materials.remove(temp_mat)

        def bake_texture(obj, uv_map_name, unwrap, use_transparency):
            bpy.context.scene.render.engine = 'CYCLES'
            bpy.context.view_layer.objects.active = obj
            obj.select_set(True)
            
            if unwrap:
                bpy.ops.object.mode_set(mode='EDIT')
                bpy.ops.mesh.select_all(action='SELECT')
                bpy.ops.uv.unwrap(method='ANGLE_BASED', margin=0.001)
                bpy.ops.object.mode_set(mode='OBJECT')
            
            bpy.context.scene.render.image_settings.file_format = 'PNG'
            bpy.context.scene.render.film_transparent = not use_transparency
            
            image = bpy.data.images.new(name=uv_map_name, width=1024, height=1024, alpha=True)
            for material_slot in obj.material_slots:
                material = material_slot.material
                nodes = material.node_tree.nodes
                image_node = nodes.new(type='ShaderNodeTexImage')
                image_node.image = image
                material.node_tree.nodes.active = image_node

            bpy.context.scene.render.bake.use_pass_direct = False
            bpy.context.scene.render.bake.use_pass_indirect = False
            bpy.ops.object.bake(type='DIFFUSE', pass_filter={'COLOR'}, use_clear=use_transparency)
            return image

        view_layer = bpy.context.view_layer
        for obj in view_layer.objects.selected:
            if obj.type == 'MESH':
                temp_materials = set_temp_material(obj)
                baked_image = bake_texture(obj, "Baked_Texture", context.scene.unwrap_uvs, context.scene.bake_props.use_transparency)
                restore_materials(obj, temp_materials)

        bpy.ops.screen.area_split(direction='HORIZONTAL', factor=0.5)
        area = bpy.context.screen.areas[-1]
        area.type = 'IMAGE_EDITOR'
        for space in area.spaces:
            if space.type == 'IMAGE_EDITOR':
                space.image = baked_image
        bpy.ops.wm.redraw_timer(type='DRAW_WIN_SWAP', iterations=1)
        return {'FINISHED'}

class OBJECT_PT_bake_texture(bpy.types.Panel):
    bl_idname = "OBJECT_PT_bake_texture"
    bl_label = "ID to Texture"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Render Tools'
    bl_order = 2

    def draw(self, context):
        layout = self.layout
        row = layout.row()
        row.scale_y = 2
        row.operator("object.bake_texture",)
        layout.prop(context.scene.bake_props, "use_transparency")
        layout.prop(context.scene, 'unwrap_uvs')
        box = layout.box()
        box.label(text="Note: You need to create first")
        box.label(text="the id colors, with the ID Tools")

classes = (
    BakeProperties,
    BakeTextureOperator,
    OBJECT_PT_bake_texture
)

def register():
    for cls in classes:
        bpy.utils.register_class(cls)
    bpy.types.Scene.unwrap_uvs = bpy.props.BoolProperty(
        name="Unwrap UVs",
        description="If checked, the object's UVs will be unwrapped",
        default=False
    )
    bpy.types.Scene.bake_props = bpy.props.PointerProperty(type=BakeProperties)

def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)
    del bpy.types.Scene.unwrap_uvs
    del bpy.types.Scene.bake_props

if __name__ == "__main__":
    register()
