import bpy
from bpy.types import Panel, Operator, PropertyGroup
import random
from bpy.props import FloatVectorProperty, BoolProperty
import os
from bpy.utils import previews

# Function to generate a random color
def random_color():
    return random.uniform(0.0, 1.0), random.uniform(0.0, 1.0), random.uniform(0.0, 1.0)

# Function to set random viewport display color to the object
def set_random_viewport_display_color(obj):
    for material_slot in obj.material_slots:
        mat = material_slot.material
        if mat is None:
            mat = bpy.data.materials.new(name="Random_Material")
            mat.use_nodes = False
            material_slot.material = mat

        color = random_color()
        mat.diffuse_color[0] = color[0]
        mat.diffuse_color[1] = color[1]
        mat.diffuse_color[2] = color[2]

# Function to set specific viewport display color to the object
def set_specific_viewport_display_color(obj, color):
    material_slot = obj.active_material
    if material_slot is None:
        mat = bpy.data.materials.new(name="Specific_Material")
        mat.use_nodes = False
        obj.active_material = mat
    else:
        mat = material_slot

    mat.diffuse_color[0] = color[0]
    mat.diffuse_color[1] = color[1]
    mat.diffuse_color[2] = color[2]

# Operator to apply random colors to the selected mesh objects
class OBJECT_OT_apply_random_colors(Operator):
    bl_idname = "object.apply_random_colors"
    bl_label = "Apply Random Colors"
    bl_description = "Apply random colors to the selected mesh objects"

    def execute(self, context):
        for obj in context.selected_objects:
            if obj.type == 'MESH':
                set_random_viewport_display_color(obj)

        return {'FINISHED'}

# Operator to apply random colors to all mesh objects in the scene
class OBJECT_OT_apply_random_colors_all_scene_objects(Operator):
    bl_idname = "object.apply_random_colors_all_scene_objects"
    bl_label = "Apply Random Colors to All Scene Objects"
    bl_description = "Apply random colors to all mesh objects in the scene"

    def execute(self, context):
        for obj in context.scene.objects:
            if obj.type == 'MESH':
                set_random_viewport_display_color(obj)

        return {'FINISHED'}

# Operator to perform a viewport render with "Flat" shading, and no overlays or outlines
class OBJECT_OT_viewport_render_flat(Operator):
    bl_idname = "object.viewport_render_flat"
    bl_label = "Render ID pass"
    bl_description = "Render the ID Pass"
    bl_order = 1

    def execute(self, context):
        # Store original settings
        original_shading = context.space_data.shading.type
        original_light = context.space_data.shading.light
        original_show_overlays = context.space_data.overlay.show_overlays
        original_object_outline = context.space_data.shading.show_object_outline
        original_cavity = context.space_data.shading.show_cavity
        original_shadow = context.space_data.shading.show_shadows
        original_dof = context.space_data.shading.use_dof

        # Apply new settings
        context.space_data.shading.type = 'SOLID'
        context.space_data.shading.light = 'FLAT'
        context.space_data.overlay.show_overlays = False
        context.space_data.shading.show_object_outline = False
        context.space_data.shading.show_cavity = False
        context.space_data.shading.show_shadows = False
        context.space_data.shading.use_dof = False

        # Perform viewport render
        if context.scene.enable_animation_render:
            bpy.ops.render.opengl(animation=True, sequencer=False)
        else:
            bpy.ops.render.opengl(write_still=True)

        # Revert to original settings
        context.space_data.shading.type = original_shading
        context.space_data.shading.light = original_light
        context.space_data.overlay.show_overlays = original_show_overlays
        context.space_data.shading.show_object_outline = original_object_outline
        context.space_data.shading.show_cavity = original_cavity
        context.space_data.shading.show_shadows = original_shadow
        context.space_data.shading.use_dof = original_dof

        # Open the rendered image in a new Image Editor window
        bpy.ops.render.view_show("INVOKE_DEFAULT")

        return {'FINISHED'}
    


    
bpy.types.Scene.create_new_material = BoolProperty(
    name="Create New Material",
    default=False,
    description="Creates a Unique ID per object istead per material type"
)


# Operator to apply specific color to the selected mesh objects
class OBJECT_OT_apply_specific_color(Operator):
    bl_idname = "object.apply_specific_color"
    bl_label = "Apply Specific Color"
    bl_description = "Apply specific Id color by Material Type of the selected object"

    def execute(self, context):
        color = context.scene.my_color
        create_new_material = context.scene.create_new_material
        for obj in context.selected_objects:
            if obj.type == 'MESH':
                if obj.active_material is not None:
                    mat = obj.active_material
                    if create_new_material:
                        # Copy the existing material
                        new_material = mat.copy()
                        # Remove the old material from the object
                        obj.data.materials.clear()
                        # Assign the new material to the object
                        obj.data.materials.append(new_material)
                        # Make the new material active
                        obj.active_material = new_material
                        # Now mat points to the new material
                        mat = new_material
                else:
                    mat = bpy.data.materials.new(name="New_Material")
                    obj.data.materials.append(mat)
                    obj.active_material = mat

                # Enable use_nodes for material
                mat.use_nodes = True
                mat.diffuse_color = (color[0], color[1], color[2], 1)

        return {'FINISHED'}



# Custom UI Panel with the buttons
class OBJECT_PT_random_colors(Panel):
    bl_label = "ID Pass"
    bl_idname = "OBJECT_PT_random_colors"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Render Tools'

    def draw(self, context):
        layout = self.layout
        
        layout = self.layout
        pcoll = preview_collections["main"]

        for icon_name in pcoll:
            my_icon = pcoll[icon_name]
            layout.template_icon(icon_value=my_icon.icon_id, scale=8)
        
        box = layout.box()
        row = layout.row()
        box.operator(OBJECT_OT_apply_random_colors.bl_idname, text="Apply Random ID Colors", icon='COLOR')
        box.operator(OBJECT_OT_apply_random_colors_all_scene_objects.bl_idname, text="Apply Random ID Colors to All", icon='SCENE_DATA')
        self.layout.separator()

        # New box for the color selection
        box = layout.box()
        box.prop(context.scene, "my_color", text="Pick ID color")
        box.operator(OBJECT_OT_apply_specific_color.bl_idname, text="Apply Specific ID Color", icon='PMARKER_ACT')
        box.prop(context.scene, "create_new_material", text="ID Color per Object")
        box = layout.box()
        box.operator(OBJECT_OT_viewport_render_flat.bl_idname, text="Render ID Pass", icon='RENDER_STILL')
        scene = context.scene
        box.prop(scene.render, "film_transparent", text="Alpha")
        box.prop(context.scene, "enable_animation_render", text="Render Animation")
        
        
# custom icon or image on top

class LoadCustomIconsOperator(bpy.types.Operator):
    bl_idname = "load.custom_icons"
    bl_label = "Load Custom Icons"
    
    def execute(self, context):
        load_icons()
        return {'FINISHED'}

preview_collections = {}

def load_icons():
    pcoll = preview_collections.get("main")
    if pcoll: bpy.utils.previews.remove(pcoll)

    pcoll = bpy.utils.previews.new()

    script_dir = os.path.dirname(os.path.realpath(__file__))
    icon_path = os.path.join(script_dir, 'icons', 'rt.png')

    my_icon_files = [icon_path]

    for icon_file in my_icon_files:
        if os.path.exists(icon_file) and icon_file.endswith('.png'):
            name = os.path.splitext(os.path.basename(icon_file))[0]
            pcoll.load(name, icon_file, 'IMAGE')

    preview_collections["main"] = pcoll

# Register the custom classes
def register():
    bpy.utils.register_class(OBJECT_OT_apply_random_colors)
    bpy.utils.register_class(OBJECT_OT_apply_random_colors_all_scene_objects)
    bpy.utils.register_class(OBJECT_OT_viewport_render_flat)
    bpy.utils.register_class(OBJECT_OT_apply_specific_color)
    bpy.utils.register_class(OBJECT_PT_random_colors)
    bpy.types.Scene.my_color = FloatVectorProperty(
        name="My Color",
        subtype='COLOR',
        default=[1.0, 1.0, 1.0],
        min=0.0, max=1.0,
        description="Color for the viewport display"
    )
    bpy.types.Scene.enable_animation_render = BoolProperty(
        name="Enable Animation Render",
        default=False,
        description="When is check it will render all the animation frames"
    )
    bpy.utils.register_class(LoadCustomIconsOperator)
    load_icons()
    

def unregister():
    bpy.utils.unregister_class(OBJECT_OT_apply_random_colors)
    bpy.utils.unregister_class(OBJECT_OT_apply_random_colors_all_scene_objects)
    bpy.utils.unregister_class(OBJECT_OT_viewport_render_flat)
    bpy.utils.unregister_class(OBJECT_OT_apply_specific_color)
    bpy.utils.unregister_class(OBJECT_PT_random_colors)
    del bpy.types.Scene.my_color
    del bpy.types.Scene.enable_animation_render
    for pcoll in preview_collections.values():
        bpy.utils.previews.remove(pcoll)
    preview_collections.clear()
    bpy.utils.unregister_class(LoadCustomIconsOperator)
    

if __name__ == "__main__":
    register()
