import bpy
from bpy.props import StringProperty, CollectionProperty, BoolProperty

def toggle_holdout_off():
    # Deselect all objects
    bpy.ops.object.select_all(action='DESELECT')

    # Iterate over all objects in the scene
    for obj in bpy.context.scene.objects:
        # Access the object's view layer settings
        for view_layer in bpy.context.scene.view_layers:
            obj_layer = view_layer.objects.get(obj.name)
            if obj_layer and obj_layer.is_holdout:
                # Select the object
                obj.select_set(True)
                # Turn off the Holdout option
                obj_layer.is_holdout = False

class OBJECT_OT_toggle_holdout(bpy.types.Operator):
    bl_idname = "object.toggle_holdout"
    bl_label = "Render Mask Pass"
    bl_description = "Select an object or objects to render selected"
    bl_options = {'REGISTER', 'UNDO'}

    def restore_state(self, scene):
        try:
            for obj in self.non_selected_objects:
                if obj and obj.name in bpy.data.objects:
                    if obj.type in {'MESH', 'CURVE'} and hasattr(obj, 'is_holdout'):
                        obj.is_holdout = False
            if hasattr(scene.render, 'film_transparent'):
                scene.render.film_transparent = self.original_film_transparent
        except Exception as e:
            self.report({'ERROR'}, f"Error in restore_state: {e}")
            print(f"Error in restore_state: {e}")
        finally:
            self.remove_handlers()

    def remove_handlers(self):
        if self.restore_state_after_render in bpy.app.handlers.render_post:
            bpy.app.handlers.render_post.remove(self.restore_state_after_render)
        if self.restore_state_after_cancel in bpy.app.handlers.render_cancel:
            bpy.app.handlers.render_cancel.remove(self.restore_state_after_cancel)

    def restore_state_after_render(self, scene, depsgraph=None):
        try:
            if self.is_animation and scene.frame_current < scene.frame_end:
                return
            self.restore_state(scene)
        except Exception as e:
            print(f"Error in restore_state_after_render: {e}")
        finally:
            self.remove_handlers()

    def restore_state_after_cancel(self, scene, depsgraph=None):
        self.restore_state(scene)

    def delayed_restore_state(self, scene):
        def restore():
            self.restore_state(scene)
            return None  # This ensures the timer is removed after execution
        return restore

    def execute(self, context):
        scene = context.scene
        selected_objects = context.selected_objects
        self.non_selected_objects = [obj for obj in scene.objects if obj not in selected_objects]

        try:
            for obj in self.non_selected_objects:
                if obj and obj.name in bpy.data.objects:
                    if obj.type in {'MESH', 'CURVE', 'EMPTY'} and hasattr(obj, 'is_holdout'):
                        obj.is_holdout = True

            self.original_film_transparent = scene.render.film_transparent
            scene.render.film_transparent = True

            bpy.app.handlers.render_post.append(self.restore_state_after_render)
            bpy.app.handlers.render_cancel.append(self.restore_state_after_cancel)

            self.is_animation = scene.render_animation
            if self.is_animation:
                bpy.ops.render.render('INVOKE_DEFAULT', animation=True)
            else:
                bpy.ops.render.render('INVOKE_DEFAULT', write_still=True)

            # Use timer instead of render handlers
            bpy.app.timers.register(self.delayed_restore_state(scene), first_interval=1.0)

            return {'FINISHED'}
        except Exception as e:
            self.report({'ERROR'}, f"Error in execute: {e}")
            print(f"Error in execute: {e}")
            self.restore_state(scene)
            return {'CANCELLED'}

class OBJECT_OT_fix_ghost_objects(bpy.types.Operator):
    bl_idname = "object.fix_ghost_objects"
    bl_label = "Fix Ghost Objects"
    bl_description = "Fix objects that have disappeared or become transparent"
    
    def execute(self, context):
        try:
            toggle_holdout_off()
            self.report({'INFO'}, "Holdout turned off for all objects")
        except Exception as e:
            self.report({'ERROR'}, f"Error in fixing ghost objects: {e}")
            print(f"Error in fixing ghost objects: {e}")
            return {'CANCELLED'}
        return {'FINISHED'}

class SelectionPresetItem(bpy.types.PropertyGroup):
    name: StringProperty(
        name="Preset Name",
        description="Name of the preset",
        default="Preset"
    )
    objects: StringProperty(
        name="Objects",
        description="List of object names in the preset",
        default=""
    )

class SELECTIONPRESETS_OT_save(bpy.types.Operator):
    bl_idname = "selectionpresets.save"
    bl_label = "Save selection preset"

    def execute(self, context):
        selected_objects = bpy.context.selected_objects
        if not selected_objects:
            self.report({'WARNING'}, "No objects selected")
            return {'CANCELLED'}
        preset = context.scene.selection_presets.add()
        preset.name = selected_objects[0].name
        for obj in selected_objects:
            preset.objects += obj.name + ','
        return {'FINISHED'}

class SELECTIONPRESETS_OT_load(bpy.types.Operator):
    bl_idname = "selectionpresets.load"
    bl_label = ""
    bl_description = "Select the preset"

    preset_name: StringProperty()

    def execute(self, context):
        bpy.ops.object.select_all(action='DESELECT')
        preset = context.scene.selection_presets[self.preset_name]
        objects = preset.objects.split(',')
        for obj in objects:
            if obj in bpy.data.objects:
                bpy.data.objects[obj].select_set(True)
        return {'FINISHED'}

class SELECTIONPRESETS_OT_delete(bpy.types.Operator):
    bl_idname = "selectionpresets.delete"
    bl_label = ""
    bl_description = "Delete the selection preset"

    preset_name: StringProperty()

    def execute(self, context):
        preset_index = context.scene.selection_presets.find(self.preset_name)
        context.scene.selection_presets.remove(preset_index)
        return {'FINISHED'}

class SELECTIONPRESETS_OT_rename(bpy.types.Operator):
    bl_idname = "selectionpresets.rename"
    bl_label = "Rename selection preset"
    bl_description = "Rename the preset"

    preset_name: StringProperty()
    new_name: StringProperty()

    def invoke(self, context, event):
        preset_index = context.scene.selection_presets.find(self.preset_name)
        self.new_name = context.scene.selection_presets[preset_index].name
        return context.window_manager.invoke_props_dialog(self)

    def execute(self, context):
        preset_index = context.scene.selection_presets.find(self.preset_name)
        context.scene.selection_presets[preset_index].name = self.new_name
        return {'FINISHED'}

class SELECTIONPRESETS_PT_panel(bpy.types.Panel):
    bl_label = "Mask Selection Presets"
    bl_idname = "SELECTIONPRESETS_PT_panel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Render Tools'
    bl_order = 3

    def draw(self, context):
        layout = self.layout
        scene = context.scene

        for preset in scene.selection_presets:
            box = layout.box()
            row = box.row()
            row.label(text=preset.name)
            op = row.operator(SELECTIONPRESETS_OT_rename.bl_idname, text="Edit")
            op.preset_name = preset.name
            op = row.operator(SELECTIONPRESETS_OT_load.bl_idname, icon="RESTRICT_SELECT_OFF")
            op.preset_name = preset.name
            op = row.operator(SELECTIONPRESETS_OT_delete.bl_idname, icon="TRASH")
            op.preset_name = preset.name
            
        row = layout.row()
        row.operator_context = 'INVOKE_DEFAULT'
        row.operator(SELECTIONPRESETS_OT_save.bl_idname)

classes = (
    SelectionPresetItem,
    SELECTIONPRESETS_OT_save,
    SELECTIONPRESETS_OT_load,
    SELECTIONPRESETS_OT_delete,
    SELECTIONPRESETS_OT_rename,
    SELECTIONPRESETS_PT_panel,
    OBJECT_OT_fix_ghost_objects,
)

class OBJECT_PT_holdout_panel(bpy.types.Panel):
    bl_label = "Mask Render Pass"
    bl_idname = "OBJECT_PT_holdout_panel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Render Tools"
    bl_order = 4

    def draw(self, context):
        layout = self.layout
        box = layout.box()
        row = layout.row()
        box.label(text="Note: Select the object or objects")
        box.label(text="to be rendered as masked")
        box.operator("object.toggle_holdout", icon='RENDER_STILL')
        box.prop(context.scene, "render_animation")
        box.operator("object.fix_ghost_objects", text="Fix Ghost Objects", icon='OUTLINER_OB_MESH')

def register():
    bpy.utils.register_class(OBJECT_OT_toggle_holdout)
    bpy.utils.register_class(OBJECT_PT_holdout_panel)
    bpy.types.Scene.render_animation = BoolProperty(name="Render Animation", default=False)
    for cls in classes:
        bpy.utils.register_class(cls)
    bpy.types.Scene.selection_presets = CollectionProperty(type=SelectionPresetItem)

def unregister():
    bpy.utils.unregister_class(OBJECT_OT_toggle_holdout)
    bpy.utils.unregister_class(OBJECT_PT_holdout_panel)
    del bpy.types.Scene.render_animation
    for cls in reversed(classes):
        bpy.utils.unregister_class(cls)
    del bpy.types.Scene.selection_presets

if __name__ == "__main__":
    register()