import bpy
from bpy.props import StringProperty, CollectionProperty


class SelectionPresetItem(bpy.types.PropertyGroup):
    name: StringProperty(
        name="Preset Name",
        description="Name of the preset",
        default="Preset")
    objects: StringProperty(
        name="Objects",
        description="List of object names in the preset",
        default="")


class SELECTIONPRESETS_OT_save(bpy.types.Operator):
    bl_idname = "selectionpresets.save"
    bl_label = "Save selection preset"

    def execute(self, context):
        selected_objects = bpy.context.selected_objects
        if not selected_objects:
            self.report({'WARNING'}, "No objects selected")
            return {'CANCELLED'}
        preset = context.scene.selection_presets.add()
        preset.name = selected_objects[0].name  # use the name of the first selected object
        for obj in selected_objects:
            preset.objects += obj.name + ','
        return {'FINISHED'}


class SELECTIONPRESETS_OT_load(bpy.types.Operator):
    bl_idname = "selectionpresets.load"
    bl_label = ""
    bl_description = "Select the preset"

    preset_name: StringProperty()

    def execute(self, context):
        bpy.ops.object.select_all(action='DESELECT')
        preset = context.scene.selection_presets[self.preset_name]
        objects = preset.objects.split(',')
        for obj in objects:
            if obj in bpy.data.objects:
                bpy.data.objects[obj].select_set(True)
        return {'FINISHED'}


class SELECTIONPRESETS_OT_delete(bpy.types.Operator):
    bl_idname = "selectionpresets.delete"
    bl_label = ""
    bl_description = "Delete the selection preset"

    preset_name: StringProperty()

    def execute(self, context):
        preset_index = context.scene.selection_presets.find(self.preset_name)
        context.scene.selection_presets.remove(preset_index)
        return {'FINISHED'}


class SELECTIONPRESETS_OT_rename(bpy.types.Operator):
    bl_idname = "selectionpresets.rename"
    bl_label = "Rename selection preset"
    bl_description = "Rename the preset"

    preset_name: StringProperty()
    new_name: StringProperty()

    def invoke(self, context, event):
        preset_index = context.scene.selection_presets.find(self.preset_name)
        self.new_name = context.scene.selection_presets[preset_index].name
        return context.window_manager.invoke_props_dialog(self)

    def execute(self, context):
        preset_index = context.scene.selection_presets.find(self.preset_name)
        context.scene.selection_presets[preset_index].name = self.new_name
        return {'FINISHED'}


class SELECTIONPRESETS_PT_panel(bpy.types.Panel):
    bl_label = "Selection Presets"
    bl_idname = "SELECTIONPRESETS_PT_panel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Selection'

    def draw(self, context):
        layout = self.layout
        scene = context.scene

        for preset in scene.selection_presets:
            box = layout.box()
            row = box.row()
            row.label(text=preset.name)
            op = row.operator(SELECTIONPRESETS_OT_rename.bl_idname, text="Edit")
            op.preset_name = preset.name
            op = row.operator(SELECTIONPRESETS_OT_load.bl_idname, icon="RESTRICT_SELECT_OFF")
            op.preset_name = preset.name
            op = row.operator(SELECTIONPRESETS_OT_delete.bl_idname, icon="TRASH")
            op.preset_name = preset.name
            
        row = layout.row()
        row.operator_context = 'INVOKE_DEFAULT'
        row.operator(SELECTIONPRESETS_OT_save.bl_idname)


classes = (
    SelectionPresetItem,
    SELECTIONPRESETS_OT_save,
    SELECTIONPRESETS_OT_load,
    SELECTIONPRESETS_OT_delete,
    SELECTIONPRESETS_OT_rename,
    SELECTIONPRESETS_PT_panel,
)


def register():
    from bpy.utils import register_class
    for cls in classes:
        register_class(cls)

    bpy.types.Scene.selection_presets = CollectionProperty(type=SelectionPresetItem)


def unregister():
    from bpy.utils import unregister_class
    for cls in reversed(classes):
        unregister_class(cls)

    del bpy.types.Scene.selection_presets


if __name__ == "__main__":
    register()
